/*
* Photo_Snapshot.js by Ron Kelley (rkelley at employees dot org)
* With original works by Roger Magnusson (roger_74 at home dot se)
* and a host of other freely available Vegas Video scripts
*
* History:
* -------   
* rel 0.61 - 28-Feb-04   Fixed minor bug with regards to Vegas Cursor.
*                        The script would try to capture snapshots if
*                        the Vegas Cursor was at end of Track.   Also,
*                        saved the cursor position before making snapshots
*                        so it can be restored when script finishes.
*                        
* rel 0.6  - 26-Feb-04   Added GUI box
*
* rel 0.5  - 25-Feb-04   Original version (used various code pieces from 
*                        jetdv and Roger Mangusson)
*/


import System;
import System.IO;
import System.Windows.Forms;
import Sony.Vegas;


// Default duration of each photo snapshot 
// (in number of Seconds [see RulerFormat above])
var Snapshot_Length = 4;

// Where is the camera sound file located?
var CameraSoundFile = "D:\\camera.wav"

// How many seconds to skip until the next photo snapshot is taken
// **MUST be longer than Snapshot_Length **
var SkipFrame_Length = 30;
var AlignImagesUnderCursor = false;
var MaxNumberPhotos = 99999;

// Type of FX to use on the background photo
var BlurType = "Sony Gaussian Blur";

// Name of program and version
var ProgramName = "Automated Photo Snapshot"
var Author = "Ron Kelley (rkelley@employees.org)"
var Version = ".6"

// Save the Original Ruler format so we can restore it later.
var oRulerFormat = Vegas.Project.Ruler.Format;

// Include the camera sound by default
var IncludeCameraSound = true;

/* ----------------------------
   Main part of the script here
   ----------------------------
*/

try {
   var skipFrame = new Timecode(SkipFrame_Length);
   var trackEnum = new Enumerator(Vegas.Project.Tracks);
   var oCursor = Vegas.Cursor;

   // var track : Track = Track(trackEnum.item());
   var track = FindSelectedTrack();

   if (!track.IsVideo() || int(track.Length.ToMilliseconds() == 0 )) { throw "Please select a Video track with at least one event" }

   if ( Vegas.Cursor >= track.Length) { throw "Cursor at end of track.  Please move timeline cursor closer to start of track"  ; }

   // Build the GUI dialog box
   var dialog = new RenderImageSequenceDialog(MaxNumberPhotos, SkipFrame_Length, Snapshot_Length, AlignImagesUnderCursor, IncludeCameraSound);
   var dialogResult = dialog.ShowDialog();

    // if the OK button was pressed...
    if (System.Windows.Forms.DialogResult.OK == dialogResult) {
        MaxNumberPhotos = int(dialog.MaxNumberPhotosBox.Text);
        Snapshot_Length = int(dialog.Snapshot_LengthBox.Text);
        SkipFrame_Length = int(dialog.SkipFrame_LengthBox.Text);

        // Set the Ruler Format for the script to "seconds"
        Vegas.Project.Ruler.Format = RulerFormat.Seconds

        var NumImagesCreated = 0;
        var skipFrameTime = new Timecode(SkipFrame_Length);
 
        // Some logic checking
        if (MaxNumberPhotos == null ) { MaxNumberPhotos = 0 } ;
        if (Snapshot_Length > SkipFrame_Length ) { throw "Error!  Photo length must be smaller than snapshot interval"}

        // Continue to create snapshots until we reach the end of the 
        // selected track or until the MaxNumberPhotos variable is reached.
        while (Vegas.Cursor < track.Length ) {
             if ( NumImagesCreated >= MaxNumberPhotos) { break ; }
             TakeSnapshot(new Timecode(Snapshot_Length), boolean(dialog.AlignImagesUnderCursorBox.Checked), boolean(dialog.IncludeCameraSoundBox.Checked));
             Vegas.Cursor = ( Vegas.Cursor + skipFrameTime);
             NumImagesCreated =  (NumImagesCreated + 1 );
        }
   // Once we are finished, show the user some information
   MessageBox.Show("Done!  Script automatically created " + NumImagesCreated + " snapshot images", ProgramName + " - Version " + Version + " - " + Author);


   // Cleanup after the photos are generated.
   // Reset the Format Ruler back to the original value
   Vegas.Project.Ruler.Format = oRulerFormat;
   Vegas.Cursor = oCursor;

   }

 } catch (e) {
   // Make sure we reset everything if we error out.
   MessageBox.Show(e,"Fatal Error");
   Vegas.Project.Ruler.Format = oRulerFormat;
}

// Thanks to Roger Magnusson for his help on some of the logic here
function TakeSnapshot(snapshotLength : Timecode, keepAtTimecode : Boolean, CameraSound : Boolean) : void
{
  for (var currentTrack : Track in Vegas.Project.Tracks)
  {
     if (currentTrack.IsVideo() == true && currentTrack.Selected == true)
     {
        for (var currentEvent : VideoEvent in currentTrack.Events)
        {
           if (currentEvent.Start <= Vegas.Cursor && (currentEvent.Start + currentEvent.Length) >= Vegas.Cursor)
           {
              // --------------------------------------------------------
              // Create the Photo and Background Tracks
              // Set the Photo Track in the first track position 
              // and the background track in the second track position
              // --------------------------------------------------------
              var PhotoTrack : VideoTrack = GetVideoTrackByName("Photo Track", 0);
              var BackgroundTrack : VideoTrack = GetVideoTrackByName("Background Track", 1);

              // Create the new Photo Event & give it a place to start
              var NewPhotoEvent : VideoEvent = VideoEvent(currentEvent.Copy(PhotoTrack,keepAtTimecode ? Vegas.Cursor : PhotoTrack.Length));
              NewPhotoEvent.ActiveTake.Offset = Vegas.Cursor - currentEvent.Start;
              NewPhotoEvent.Length = snapshotLength;
        
              var BackgroundImage : VideoEvent = VideoEvent(currentEvent.Copy(BackgroundTrack,keepAtTimecode ? Vegas.Cursor : BackgroundTrack.Length));
              BackgroundImage.ActiveTake.Offset = Vegas.Cursor - currentEvent.Start;
              BackgroundImage.Length = snapshotLength;

              // Add a "Flash" Fade in Transition to simulate flash photography
              // Use the "Dissolve" Fade our Transition
              var PhotoFadeInTx = CreateTransitionEffect(/Flash/);
              NewPhotoEvent.FadeIn.Transition = PhotoFadeInTx;
              NewPhotoEvent.FadeIn.Length = new Timecode(250);

              // Uncomment the following lines if you want a Fade Out transition on each photo
              // var PhotoFadeOutTx = CreateTransitionEffect(/Dissolve/);
              // NewPhotoEvent.FadeOut.Transition = PhotoFadeOutTx;
              // PhotoFadeOutTx.Preset = "Additive Dissolve";
              // NewPhotoEvent.FadeOut.Length = new Timecode(500);

              //Add a fade-in and fade-out transition for the Background
              var BackgroundFadeIn = CreateTransitionEffect(/Dissolve/);
              BackgroundImage.FadeIn.Transition = BackgroundFadeIn;
              BackgroundImage.FadeIn.Length = new Timecode(250);

              // Uncomment the following lines if you want a Fade Out transition on the background photo
              // var BackgroundFadeOut = CreateTransitionEffect(/Dissolve/);
              // BackgroundImage.FadeOut.Transition = BackgroundFadeOut;
              // BackgroundFadeOut.Preset = "Additive Dissolve";
              // BackgroundImage.FadeOut.Length = new Timecode(500);

              // Add a velocity envelope on the Photo track and make it "0" speed (freeze frame)
              EnvelopePoint(Envelope(NewPhotoEvent.Envelopes[NewPhotoEvent.Envelopes.Add(
              new Envelope(EnvelopeType.Velocity))]).Points.GetPointAtX(new Timecode(0))).Y = 0;

              // Add a velocity envelope on the Background track and make it "0" speed (freeze frame)
              EnvelopePoint(Envelope(BackgroundImage.Envelopes[BackgroundImage.Envelopes.Add(
              new Envelope(EnvelopeType.Velocity))]).Points.GetPointAtX(new Timecode(0))).Y = 0;

              // Create some keyevents for the PhotoEvent and the Background Event
              var NewPhotoEventKey1 = NewPhotoEvent.VideoMotion.Keyframes(0) ;
              var NewPhotoEventKey2 = new VideoMotionKeyframe(new Timecode(1000));

              var BackgroundEventKey1 = BackgroundImage.VideoMotion.Keyframes(0);
              // var BackgroundEventKey2 = new VideoMotionKeyframe(new Timecode(1000));

              // add the new key frames (keyframe1 already added by system)
              NewPhotoEvent.VideoMotion.Keyframes.Add(NewPhotoEventKey2);

              // Move the Photo Event
              var ZoomSlidePercent = 200 ;
              var NewPhotoEventScale1 = new VideoMotionVertex(float(ZoomSlidePercent) / 100.0, float(ZoomSlidePercent) / 100.0);
              NewPhotoEventKey1.ScaleBy(NewPhotoEventScale1);
              NewPhotoEventKey1.RotateBy(-.1);
              var NewPhotoEventScale2= new VideoMotionVertex(float(ZoomSlidePercent) / 175.0, float(ZoomSlidePercent) / 175.0);
              NewPhotoEventKey2.ScaleBy(NewPhotoEventScale2);

              // Add the Blur Effect to the Background image
              var plugIn = Vegas.VideoFX.GetChildByName(BlurType);
              var effect = new Effect(plugIn);
              BackgroundImage.Effects.Add(effect);


               // If we want the camera sound, add it to the camera track
              if ( CameraSound == true ) { 
                  var CameraTrack : AudioTrack = GetAudioTrackByName("Camera Sound",2); 
                  var camera_sound = new Media(CameraSoundFile);
                  if (!camera_sound.IsValid()) throw "media file does not exist: " + CameraSoundFile;
                  var stream,newEvent;
                  stream = camera_sound.Streams.GetItemByMediaType(MediaType.Audio, 0);
                  newEvent = new AudioEvent(Vegas.Cursor,stream.Length);
                  CameraTrack.Events.Add(newEvent);
                  newEvent.Start = NewPhotoEvent.Start;
                  var take = new Take(stream);
                  newEvent.Takes.Add(take);
               }

            break;
            }
         }
     break;
     }
  }
}

function FindSelectedTrack() : Track {
  trackEnum = new Enumerator(Vegas.Project.Tracks);
  while (!trackEnum.atEnd()) {
    var track : Track = Track(trackEnum.item());

    if (track.Selected) {
      return track;
    }
    trackEnum.moveNext();
  }
  return null;
}


function GetVideoTrackByName(trackName : String, trackNumber : int  ) : VideoTrack
{
    for (var currentTrack : Track in Vegas.Project.Tracks)
    {
        if (currentTrack.Name == trackName && currentTrack.IsVideo() == true)
        {
            return VideoTrack(currentTrack);
        }
    }

    // var newTrack : VideoTrack = new VideoTrack(Vegas.Project.Tracks.Count + 1, trackName);
    var newTrack : VideoTrack = new VideoTrack(trackNumber, trackName);
    Vegas.Project.Tracks.Add(newTrack);

    return newTrack;
}

function GetAudioTrackByName(trackName : String, trackNumber : int) : AudioTrack
{
    for (var currentTrack : Track in Vegas.Project.Tracks)
    {
        if (currentTrack.Name == trackName && currentTrack.IsAudio() == true)
        {
            return AudioTrack(currentTrack);
        }
    }

    var newTrack : AudioTrack = new AudioTrack(trackNumber, trackName);
    Vegas.Project.Tracks.Add(newTrack);

    return newTrack;
}


function FindPlugInNode(rootNode : PlugInNode, nameRegExp : RegExp) : PlugInNode {
    if (null != rootNode.Name.match(nameRegExp)) {
        return rootNode;
    } else {
        var children : Enumerator = new Enumerator(rootNode);
        while (!children.atEnd()) {
            var childNode : PlugInNode = PlugInNode(children.item())
            var childMatch : PlugInNode = FindPlugInNode(childNode, nameRegExp);
            if (null != childMatch) {
                return childMatch;
            }
            children.moveNext();
        }
        return null;
    }
}

function CreateTransitionEffect(nameRegExp : RegExp) : Effect {
    var plugIn = FindPlugInNode(Vegas.Transitions, nameRegExp);
    if (null == plugIn)
        throw "failed to find plug-in";
    return new Effect(plugIn);
}

function CreateGeneratedMedia(generatorName, presetName) {
  var generator = Vegas.Generators.GetChildByName(generatorName);
  var media = new Media(generator, presetName);
  if (!media.IsValid()) {
      throw "failed to create media; " + generatorName + " (" + presetName + ")";
  }
  return media;
}



// Form subclass that is the dialog box for this script
// The following code borrowed from other public-domain code
class RenderImageSequenceDialog extends Form {
    var browseButton;
    var MaxNumberPhotosBox;
    var SkipFrame_LengthBox;
    var Snapshot_LengthBox;
    var AlignImagesUnderCursorBox;
    var IncludeCameraSoundBox;

    function RenderImageSequenceDialog(MaxNumberPhotos, SkipFrame_Length, Snapshot_Length, AlignImagesUnderCursor, IncludeCameraSound) {

        this.Text = "Automatic Photo Snapshot - Ron Kelley - version 0.60 ";
        this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
        this.MaximizeBox = false;
        this.StartPosition = FormStartPosition.CenterScreen;
        this.Width = 475;
        this.Height = 380;


        MaxNumberPhotosBox = addTextControl("Number of Stills to capture", 30, 180, 20, MaxNumberPhotos.ToString());
        Snapshot_LengthBox = addTextControl("            Photo Length (sec)", 30, 180, 40, Snapshot_Length.ToString());
        SkipFrame_LengthBox = addTextControl("     Snapshot Interval (sec)", 30, 180, 60, SkipFrame_Length.ToString());
        AlignImagesUnderCursorBox = addCheckbox("Align Image to timeline cursor:", 1, 240, 20);
        IncludeCameraSoundBox = addCheckbox("           Include Camera Sound:", 1, 240, 40);

        var buttonWidth = 100;
        var buttonHeight = 100;
        var buttonTop = 120;
        var okButton = new Button();
        okButton.Text = "OK";
        okButton.Left = this.Width - (3*(buttonWidth+10));
        okButton.Top = buttonTop;
        okButton.Height = okButton.Font.Height + 12;
        okButton.DialogResult = System.Windows.Forms.DialogResult.OK;
        AcceptButton = okButton;
        Controls.Add(okButton);

        var cancelButton = new Button();
        cancelButton.Text = "Cancel";
        cancelButton.Left = this.Width - (1*(buttonWidth+10));
        cancelButton.Top = buttonTop;
        cancelButton.Height = cancelButton.Font.Height + 12;
        cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
        CancelButton = cancelButton;
        Controls.Add(cancelButton);

        var titleHeight = this.Height - this.ClientSize.Height;
        this.Height = titleHeight + okButton.Bottom + 8;
    }

    function addTextControl(labelName, left, width, top, defaultValue) {
        var label = new Label();
        label.AutoSize = true;
        label.Text = labelName + ":";
        label.Left = left;
        label.Top = top + 4;
        Controls.Add(label);

        var textbox = new TextBox();
        textbox.Multiline = false;
        textbox.Left = label.Right;
        textbox.Top = top;
        textbox.Width = width - (label.Width);
        textbox.Text = defaultValue;
        Controls.Add(textbox);
        return textbox;
    }


    function addCheckbox(labelName, checkbox, left, top) {
        var label = new Label();
        label.AutoSize = true;
        label.Text = labelName;
        label.Left = left;
        label.Top = top + 4;
        Controls.Add(label);

        if (checkbox == 1) {
           var checkbox = new CheckBox();
           checkbox.Left = label.Right;
           checkbox.Width = 36;
           checkbox.Top = top;
           Controls.Add(checkbox);
         return checkbox;
        }
     }

}


/* --------------------
     END OF SCRIPT 
   --------------------
*/
