/**
 * Name: MultiCamDissolve
 * Description: Multi-Camera Mixing with Dissolves between clips
 * Written by: Edward Troxel
 * Copyright: 2003
 * Company: JET Digital Video
 *
 * This script will simplify the editing of multi-camera events by moving the selected
 * segment to a "Master" track and automatically add a dissolve into the previous clip.
 *
 * To use this script, add as many cameras as desired and get them in sync.
 * Next, add another video track and name that video track "Master". This should be the
 * topmost track unless you need additional tracks for titles or special effects. It MUST
 * be above the individual camera tracks and MUST be named "Master". If an event is too
 * close to the beginning of the track, it will not be extended and will not search for
 * a previous event to also extend.
 * 
 **/

import System;
import System.IO;
import System.Windows.Forms;
import Sony.Vegas;


try {
  // Length to extend the clips
  // NOTE: the actual dissolve will be DOUBLE this length!!!
  // "00:00:00:08" = 8 frames which, when doubled, equals (roughly) 1/2 second
  var DefaultDissolveLength = "00:00:00:08"

  // Find the Master track
  DeselectAllTracks()
  var Mastertrack = FindMasterTrack();
  if (null == Mastertrack)
      throw "no selected track";
  Mastertrack.Selected = true;

  //Find the selected event
  var track = FindSelectedTrack();
  if (null == track)
      throw "no selected track";

  var evnt = FindSelectedEvent();
  if (null == evnt)
      throw "no selected event";

  // Move the event to the Master track
  evnt.Track = Mastertrack;

  // Now, extend the beginning of this clip backward 8 frames
  var NewStart = new Timecode(evnt.Start.ToString());
  var NewLength = new Timecode(evnt.Length.ToString());
  var ZeroTime = new Timecode();
  var LenOffset = new Timecode(DefaultDissolveLength);

  // Don't extend the length of a clip to close to the beginning of the track
  if (NewStart > ZeroTime) {
      NewStart = NewStart - LenOffset;
      NewLength = NewLength + LenOffset;

      evnt.AdjustStartLength(NewStart, NewLength, false);

      // Find the Previous event (event to the left of the selected event)
      var preevnt = FindPreviousEvent();
      if (null == preevnt)
          throw "no selected event";

      // Now, extend the END of the previous clip forward 8 frames
      NewStart = new Timecode(preevnt.Start.ToString());
      NewLength = new Timecode(preevnt.Length.ToString());

      NewLength = NewLength + LenOffset;

      preevnt.AdjustStartLength(NewStart, NewLength, true);
  }



} catch (e) {
    MessageBox.Show(e);
}


function FindSelectedTrack() : Track {
  var trackEnum = new Enumerator(Vegas.Project.Tracks);
  while (!trackEnum.atEnd()) {
    var track : Track = Track(trackEnum.item());
    if (track.Selected) {
        return track;
    }
    trackEnum.moveNext();
  }
  return null;
}

function FindMasterTrack() : Track {
  var trackEnum = new Enumerator(Vegas.Project.Tracks);
  while (!trackEnum.atEnd()) {
    var track : Track = Track(trackEnum.item());
    if (track.Name == "Master") {
        return track;
    }
    trackEnum.moveNext();
  }
  return null;
}

function FindSelectedEvent() : TrackEvent {
  var trackEnum = new Enumerator(Vegas.Project.Tracks);
  while (!trackEnum.atEnd()) {
    var track : Track = Track(trackEnum.item());
    var eventEnum = new Enumerator(track.Events);
    while (!eventEnum.atEnd()) {
      var evnt : TrackEvent = TrackEvent(eventEnum.item());
      if (evnt.Selected) {
          return evnt;
      }
      eventEnum.moveNext();
    }
    trackEnum.moveNext();
  }
  return null;
}

function FindPreviousEvent() : TrackEvent {
  var trackEnum = new Enumerator(Vegas.Project.Tracks);
  while (!trackEnum.atEnd()) {
    var track : Track = Track(trackEnum.item());
    var eventEnum = new Enumerator(track.Events);
    var preevnt : TrackEvent = TrackEvent(eventEnum.item());
    while (!eventEnum.atEnd()) {
      var evnt : TrackEvent = TrackEvent(eventEnum.item());
      if (evnt.Selected) {
          return preevnt;
      }
      preevnt = evnt;
      eventEnum.moveNext();
    }
    trackEnum.moveNext();
  }
  return null;
}

function DeselectAllTracks() : Track {
  var trackEnum = new Enumerator(Vegas.Project.Tracks);
  while (!trackEnum.atEnd()) {
    var track : Track = Track(trackEnum.item());
    track.Selected = false;
    trackEnum.moveNext();
  }
  return null;
}

