/*
* PhotoSnapshot.js by Ron Kelley (rkelley at employees dot org)
* With original works by Roger Magnusson (roger_74 at home dot se)
* and a host of other freely available Vegas Video scripts
*
* History:
* -------   
* rel 0.61  - 26-Feb-04  Fixed bug with the NewPhotoEvent function.
*                        Thanks to Roger Mangusson for his help in
*                        finding/correcting the problem.

*
* rel 0.60 - 26-Feb-04   Fixed minor bug with Vegas Cursor.
*
* rel 0.50 - 25-Feb-04   Original version (used various code pieces from 
*                        jetdv and Roger Mangusson)
*/

import System;
import System.IO;
import System.Windows.Forms;
import Sony.Vegas;


// Name of program and version
var ProgramName = "Photo Snapshot"
var Author = "Ron Kelley (rkelley@employees.org)"
var Version = ".61"

// Where is the camera sound file located?
//var CameraSoundFile;
var CameraSoundFile = "g:\\camera.wav";
var enableSound : boolean = true;


// Save the Original Ruler format so we can restore it later.
// Set the Ruler Format for the script to "seconds"
var oRulerFormat = Vegas.Project.Ruler.Format;
Vegas.Project.Ruler.Format = RulerFormat.Seconds;

// ----------------
// MAIN STARTS HERE
// ----------------

// If you want the snapshot to appear under the cursor instead of
// stacked in a row, change the last parameter to true. 
TakeSnapshot(new Timecode(4000), true);

// Restore the original ruler format
Vegas.Project.Ruler.Format = oRulerFormat;

// ---END OF SCRIPT---



function TakeSnapshot(snapshotLength : Timecode, keepAtTimecode : Boolean) : void {

    for (var currentTrack : Track in Vegas.Project.Tracks) {

        if (currentTrack.IsVideo() == true && currentTrack.Selected == true) {

            for (var currentEvent : VideoEvent in currentTrack.Events) {

                if (currentEvent.Start <= Vegas.Cursor && (currentEvent.Start + currentEvent.Length) >= Vegas.Cursor) {

                    if ( !File.Exists(CameraSoundFile) )  
                    {  MessageBox.Show("Cannot locate Camera Sound File.  Please edit script and specify correct location of camera sound file.", "Error"); 
                       enableSound = false;
                    }
                    var photoTrack : VideoTrack = GetVideoTrackByName("Photo Track", 0);
                    var BackgroundTrack : VideoTrack = GetVideoTrackByName("Background Track", 1);

                    // Create the new Photo Event & give it a place to start
                    var NewPhotoEvent : VideoEvent = VideoEvent(currentEvent.Copy(photoTrack,keepAtTimecode ? Vegas.Cursor : photoTrack.Length));
                    NewPhotoEvent.ActiveTake.Offset = Vegas.Cursor - currentEvent.Start + currentEvent.ActiveTake.Offset;
                    NewPhotoEvent.Length = snapshotLength;
        
                    var BackgroundImage : VideoEvent = VideoEvent(currentEvent.Copy(BackgroundTrack,keepAtTimecode ? Vegas.Cursor : BackgroundTrack.Length));
                    BackgroundImage.ActiveTake.Offset = Vegas.Cursor - currentEvent.Start + currentEvent.ActiveTake.Offset;
                    BackgroundImage.Length = snapshotLength;

                   // Add a "Flash" Fade in Transition to simulate flash photography
                   // Use the "Dissolve" Fade our Transition
                    var PhotoFadeInTx = CreateTransitionEffect(/Flash/);
                    NewPhotoEvent.FadeIn.Transition = PhotoFadeInTx;
                    NewPhotoEvent.FadeIn.Length = new Timecode(250);
                    var PhotoFadeOutTx = CreateTransitionEffect(/Dissolve/);
                    NewPhotoEvent.FadeOut.Transition = PhotoFadeOutTx;
                    PhotoFadeOutTx.Preset = "Additive Dissolve";
                    NewPhotoEvent.FadeOut.Length = new Timecode(500);

                   //Add a fade-in and fade-out transition for the Background
                    var BackgroundFadeIn = CreateTransitionEffect(/Dissolve/);
                    BackgroundImage.FadeIn.Transition = BackgroundFadeIn;
                    BackgroundImage.FadeIn.Length = new Timecode(250);
                    var BackgroundFadeOut = CreateTransitionEffect(/Dissolve/);
                    BackgroundImage.FadeOut.Transition = BackgroundFadeOut;
                    BackgroundFadeOut.Preset = "Additive Dissolve";
                    BackgroundImage.FadeOut.Length = new Timecode(500);

                   // Add a velocity envelope on the Photo track and make it "0" speed (freeze frame)
                    EnvelopePoint(Envelope(NewPhotoEvent.Envelopes[NewPhotoEvent.Envelopes.Add(
                    new Envelope(EnvelopeType.Velocity))]).Points.GetPointAtX(new Timecode(0))).Y = 0;

                   // Add a velocity envelope on the Background track and make it "0" speed (freeze frame)
                    EnvelopePoint(Envelope(BackgroundImage.Envelopes[BackgroundImage.Envelopes.Add(
                    new Envelope(EnvelopeType.Velocity))]).Points.GetPointAtX(new Timecode(0))).Y = 0;

                   // Add some motion to the Photo frame
                   // Create some keyevents for the PhotoEvent and the Background Event
                   var NewPhotoEventKey1 = NewPhotoEvent.VideoMotion.Keyframes(0) ;
                   var NewPhotoEventKey2 = new VideoMotionKeyframe(new Timecode(1000));

                   var BackgroundEventKey1 = BackgroundImage.VideoMotion.Keyframes(0);
                   // var BackgroundEventKey2 = new VideoMotionKeyframe(new Timecode(1000));

                   // add the new key frames (keyframe1 already added by system)
                   NewPhotoEvent.VideoMotion.Keyframes.Add(NewPhotoEventKey2);

                   // Move the Photo Event
                   var ZoomSlidePercent = 200 ;
                   var NewPhotoEventScale1 = new VideoMotionVertex(float(ZoomSlidePercent) / 100.0, float(ZoomSlidePercent) / 100.0);
                   NewPhotoEventKey1.ScaleBy(NewPhotoEventScale1);
                   NewPhotoEventKey1.RotateBy(-.1);
                   var NewPhotoEventScale2= new VideoMotionVertex(float(ZoomSlidePercent) / 175.0, float(ZoomSlidePercent) / 175.0);
                   NewPhotoEventKey2.ScaleBy(NewPhotoEventScale2);

                  // Add the Gaussian Blur to the Background image
                   var plugIn = Vegas.VideoFX.GetChildByName("Sony Gaussian Blur");
                   var effect = new Effect(plugIn);
                   BackgroundImage.Effects.Add(effect);

                  // Add the camera sound to the camera track
                  if (enableSound) {
                     var CameraTrack : AudioTrack = GetAudioTrackByName("Camera Sound",2);
                     var camera_sound = new Media(CameraSoundFile);
                     var stream,newEvent;
                     stream = camera_sound.Streams.GetItemByMediaType(MediaType.Audio, 0);
                     newEvent = new AudioEvent(Vegas.Cursor,stream.Length);
                     CameraTrack.Events.Add(newEvent);
                     var take = new Take(stream);
                     newEvent.Takes.Add(take); 
                  }

                    break;
                }
            }
            break;
        }
    }
}

function GetVideoTrackByName(trackName : String, trackNumber : int  ) : VideoTrack
{
    for (var currentTrack : Track in Vegas.Project.Tracks)
    {
        if (currentTrack.Name == trackName && currentTrack.IsVideo() == true)
        {
            return VideoTrack(currentTrack);
        }
    }

    // var newTrack : VideoTrack = new VideoTrack(Vegas.Project.Tracks.Count + 1, trackName);
    var newTrack : VideoTrack = new VideoTrack(trackNumber, trackName);
    Vegas.Project.Tracks.Add(newTrack);

    return newTrack;
}

function GetAudioTrackByName(trackName : String, trackNumber : int) : AudioTrack
{
    for (var currentTrack : Track in Vegas.Project.Tracks)
    {
        if (currentTrack.Name == trackName && currentTrack.IsAudio() == true)
        {
            return AudioTrack(currentTrack);
        }
    }

    var newTrack : AudioTrack = new AudioTrack(trackNumber, trackName);
    Vegas.Project.Tracks.Add(newTrack);

    return newTrack;
}


function FindPlugInNode(rootNode : PlugInNode, nameRegExp : RegExp) : PlugInNode {
    if (null != rootNode.Name.match(nameRegExp)) {
        return rootNode;
    } else {
        var children : Enumerator = new Enumerator(rootNode);
        while (!children.atEnd()) {
            var childNode : PlugInNode = PlugInNode(children.item())
            var childMatch : PlugInNode = FindPlugInNode(childNode, nameRegExp);
            if (null != childMatch) {
                return childMatch;
            }
            children.moveNext();
        }
        return null;
    }
}

function CreateTransitionEffect(nameRegExp : RegExp) : Effect {
    var plugIn = FindPlugInNode(Vegas.Transitions, nameRegExp);
    if (null == plugIn)
        throw "failed to find plug-in";
    return new Effect(plugIn);
}

function CreateGeneratedMedia(generatorName, presetName) {
  var generator = Vegas.Generators.GetChildByName(generatorName);
  var media = new Media(generator, presetName);
  if (!media.IsValid()) {
      throw "failed to create media; " + generatorName + " (" + presetName + ")";
  }
  return media;
}

